//
//  PreferencesController.m
//  Paparazzi!
//
//  Created by Wevah on 2005.08.22.
//  Copyright 2005 Derailer. All rights reserved.
//
// Copyright (C) 2004-2005 Nate Weaver (Wevah)
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "PreferencesController.h"
#import "PaparazziDefaultsConstants.h"

static PreferencesController *kController = nil;

@implementation PreferencesController

- (id)initWithWindow:(NSWindow *)window {
	if (!kController && (self = [super initWithWindow:window])) {
		[self setWindowFrameAutosaveName:@"Preferences"];
		kController = self;
	}
	
	return kController;
}

+ (PreferencesController *)controller {
	if (!kController)
		[[PreferencesController alloc] initWithWindowNibName:@"Preferences"];
	
	return kController;
}

- (void)awakeFromNib {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	[filenameFormatField setStringValue:[defaults objectForKey:kPaparazziFilenameFormatKey]];
	[maxHistoryField setIntValue:[defaults integerForKey:kPaparazziMaxHistoryKey]];
	[GMTSwitch setState:[defaults boolForKey:kPaparazziUseGMTKey] ? NSOnState : NSOffState];
	[thumbnailSuffixField setStringValue:[defaults objectForKey:kPaparazziThumbnailSuffixKey]];
}

#pragma mark -

- (IBAction)setFilenameFormat:(id)sender {
	[[NSUserDefaults standardUserDefaults] setObject:[sender stringValue] forKey:kPaparazziFilenameFormatKey];
}

- (IBAction)setMaxHistory:(id)sender {
	if (![sender intValue])
		[sender setIntValue:0];
	
	[[NSUserDefaults standardUserDefaults] setInteger:[sender intValue] forKey:kPaparazziMaxHistoryKey];
}

- (IBAction)setUseGMT:(id)sender {
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] == NSOnState forKey:kPaparazziUseGMTKey];
}

- (IBAction)setThumbnailSuffix:(id)sender {
	[[NSUserDefaults standardUserDefaults] setObject:[sender stringValue] forKey:kPaparazziThumbnailSuffixKey];
}

- (void)controlTextDidChange:(NSNotification *)note {
	NSControl *control = [note object];
	NSFormatter *formatter = [control formatter];
	NSText *editor = [[note userInfo] objectForKey:@"NSFieldEditor"];
	
	if (!formatter || [formatter getObjectValue:nil forString:[editor string] errorDescription:nil]) {
		// Don't try to set the default if the formatter says NO, so the field doesn't end editing and clear itself.
		if (control == filenameFormatField)
			[self setFilenameFormat:control];
		if (control == thumbnailSuffixField)
			[self setThumbnailSuffix:control];
		else if (control == maxHistoryField)
			[self setMaxHistory:control];
		else
			NSLog(@"Prefs error.");
	}
}

@end
